/*
** Module   : ComboBoxWidget.java
** Abstract : server side combo box widget implementation
**
** Copyright (c) 2005-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 NVS 20051026   @23483 Created initial version.
** 002 SIY 20050209   @24433 Major refactoring.
** 003 SIY 20050209   @24494 Added missing setInnerLines method.
** 004 GES 20060803   @28368 Some cleanup and refactoring.
** 005 GES 20080320   @37583 Added TYPE attribute.
** 006 SIY 20111101          Updated imports.
** 007 CA  20130221          Refactored usage of TYPE attribute.
** 008 CA  20130222          Added NPE protection for cases when a widget is used before is 
**                           attached to a frame (after a CREATE WIDGET statement is used).
** 009 CA  20130927          Added LegacyResource annotation, to determine resource type.
** 010 CA  20131115          Changes related to properly instantiate dynamic resources.
** 011 CA  20140926          Added shared widget configuration support, to allow GUI/ChUI
**                           concrete implementation for the same widget ID.  Refactored the
**                           widget configuration classes: all fields were made public; these
**                           classes need to be as dumb as possible, all logic related to
**                           setting/getting a certain field should be at the widget or at the
**                           caller. Refs #2254
** 012 SVL 20141102          Added MAX-CHARS and AUTO-ZAP attributes.
** 013 GES 20141124          Added INNER-LINES interface, NUM-ITEMS attribute and the
**                           SET-SELECTION() method support.
** 014 IAS 20150123          Implemented DELETE() method
** 015 IAS 20150209          Fixed behavior to be compatible with 4GL.
** 016 IAS 20150225          Fixed support for the LIST-ITEMS and LIST-ITEM-PAIRS attributes
** 017 EVL 20150528          Adding mode setters implementation.
*/ 

package com.goldencode.p2j.ui;

import java.util.*;

import com.goldencode.p2j.util.*;

/**
 * Server side combo box widget implementation.
 */
@LegacyResource(resource = LegacyResource.COMBO_BOX)
public class ComboBoxWidget
extends ControlSetEntity<ComboBoxConfig>
implements AutoZapElement,
           MaxCharsElement,
           NumItems,
           SortedElement,
           InnerLines
{
   /**
    * Default constructor.
    */
   public ComboBoxWidget()
   {
      this(false);
   }
   
   /**
    * Default constructor.
    * 
    * @param   dynamic
    *          Flag indicating if this is a static or dynamic resource.
    */
   public ComboBoxWidget(boolean dynamic)
   {
      super(dynamic, new ComboBoxConfig());
   }
   
   /**
    * Set DROP-DOWN-LIST mode for the widget. Has no effect in character mode
    * interface.
    * 
    * @param   b
    *          <code>true</code> enables mode, <code>false</code> is
    *          ignored.
    */
   public void setModeDropDownList(boolean b)
   {
      // this mode should work in all UI cases
      if (b && config.mode != ComboBoxConfig.Mode.DROP_DOWN_LIST)
      {
         config.mode = ComboBoxConfig.Mode.DROP_DOWN_LIST;
         pushScreenDefinition();
      }
   }

   /**
    * Set DROP-DOWN mode for the widget. Has no effect in character mode
    * interface.
    * 
    * @param   b
    *          <code>true</code> enables mode, <code>false</code> is
    *          ignored.
    */
   public void setModeDropDown(boolean b)
   {
      // this is valid only in GUI
      if (b && !LogicalTerminal.isChui() && config.mode != ComboBoxConfig.Mode.DROP_DOWN)
      {
         config.mode = ComboBoxConfig.Mode.DROP_DOWN;
         pushScreenDefinition();
      }
   }
   
   /**
    * Set SIMPLE mode for the widget. Has no effect in character mode
    * interface.
    * 
    * @param   b
    *          <code>true</code> enables mode, <code>false</code> is
    *          ignored.
    */
   public void setModeSimple(boolean b)
   {
      // this is valid only in GUI
      if (b && !LogicalTerminal.isChui() && config.mode != ComboBoxConfig.Mode.SIMPLE)
      {
         config.mode = ComboBoxConfig.Mode.SIMPLE;
         pushScreenDefinition();
      }
   }

   /**
    * Get the NUM-ITEMS attribute.
    *
    * @return   The numebr of items in the list.
    */
   public integer getNumItems()
   {
      UnimplementedFeature.missing("NUM-ITEMS attribute getter");
      return new integer();
   }
   
   /**
    * Selects text in a widget between start and end points.
    * 
    * @param    start
    *           Starting offset.
    * @param    end
    *           Ending offset.
    *
    * @return   <code>true</code> if the operation succeeded.
    */
   public logical setSelection(int64 start, int64 end)
   {
      UnimplementedFeature.missing("SET-SELECTION method");
      return new logical();
   }
   
   /**
    * Selects text in a widget between start and end points.
    * 
    * @param    start
    *           Starting offset.
    * @param    end
    *           Ending offset.
    *
    * @return   <code>true</code> if the operation succeeded.
    */
   public logical setSelection(int64 start, int end)
   {
      UnimplementedFeature.missing("SET-SELECTION method");
      return new logical();
   }
   
   /**
    * Selects text in a widget between start and end points.
    * 
    * @param    start
    *           Starting offset.
    * @param    end
    *           Ending offset.
    *
    * @return   <code>true</code> if the operation succeeded.
    */
   public logical setSelection(int start, int64 end)
   {
      UnimplementedFeature.missing("SET-SELECTION method");
      return new logical();
   }
   
   /**
    * Selects text in a widget between start and end points.
    * 
    * @param    start
    *           Starting offset.
    * @param    end
    *           Ending offset.
    *
    * @return   <code>true</code> if the operation succeeded.
    */
   public logical setSelection(int start, int end)
   {
      UnimplementedFeature.missing("SET-SELECTION method");
      return new logical();
   }
   
   /**
    * Gets the value of the INNER-LINES attribute which defines the number of text lines
    * in the widget.
    * 
    * @return   The number of lines of text.
    */
   public integer getInnerLines()
   {
      return new integer(config.innerLines);
   }
   
   /**
    * Sets the value of the INNER-LINES attribute which defines the number of text lines
    * in the widget.
    * 
    * @param    lines
    *           The new number of lines.
    */
   public void setInnerLines(double lines)
   {
      config.innerLines = (int) lines;
      pushScreenDefinition();
   }

   /**
    * Sets the value of the INNER-LINES attribute which defines the number of text lines
    * in the widget.
    * 
    * @param    lines
    *           The new number of lines.
    */
   public void setInnerLines(NumberType lines)
   {
      if (lines == null || lines.isUnknown())
      {
         UnimplementedFeature.todo("INNER-LINES attribute unknown behavior is not known.");
         return;
      }
   
      setInnerLines(lines.intValue());
   }

   /**
    * Sets the DISABLE-AUTO-ZAP option.
    * 
    * @param    disableAutoZap
    *           <code>true</code> if the option is set to ON.
    */
   public void setDisableAutoZap(boolean disableAutoZap)
   {
      config.autoZap = !disableAutoZap;
      pushScreenDefinition();
   }

   /**
    * Set AUTO-ZAP attribute.
    * 
    * @param    value
    *           New value for the attribute.
    */
   public void setAutoZap(boolean value)
   {
      config.autoZap = value;
      pushScreenDefinition();
   }

   /**
    * Set value of the MAX-CHARS attribute.
    *
    * @param    maxChars
    *           Value of the attribute.
    */
   public void setMaxChars(double maxChars)
   {
      setMaxChars(new decimal(maxChars));
   }

   /**
    * Set value of the MAX-CHARS attribute.
    *
    * @param    maxChars
    *           Value of the attribute.
    */
   public void setMaxChars(NumberType maxChars)
   {
      if (maxChars.isUnknown())
         return;

      config.maxChars = maxChars.intValue();
      pushScreenDefinition();
   }

   /**
    * Get value of the MAX-CHARS attribute.
    *
    * @return   value of the attribute.
    */
   public integer getMaxChars()
   {
      return new integer(config.maxChars);
   }

   /**
    * Set value of the AUTO-ZAP attribute.
    *
    * @param    autoZap
    *           Value of the attribute.
    */
   public void setAutoZap(logical autoZap)
   {
      if (autoZap.isUnknown())
         return;

      setAutoZap(autoZap.booleanValue());
   }

   /**
    * Get value of the AUTO-ZAP attribute.
    *
    * @return   value of the attribute.
    */
   public logical isAutoZap()
   {
      return new logical(config.autoZap);
   }

   /**
    * Implements the DELETE() widget method, which removes the list item or
    * items specified.
    * 
    * @param itemList
    *           The item or items to delete.
    * 
    * @return <code>true</code> if the method succeeds.
    */
   @Override
   public logical delete(character itemList)
   {
      return delete(itemList, COMPARE_VALUE_IGNORE_CASE, true);
   }

   /**
    * Implements the DELETE() widget method, which removes the list item or
    * items specified.
    * 
    * @param itemList
    *           The item or items to delete.
    * 
    * @return <code>true</code> if the method succeeds.
    */
   public logical delete(String itemList)
   {
      return delete(itemList, COMPARE_VALUE_IGNORE_CASE, true);
   }
   
   /**
    * Remove trailing whitespaces
    * 
    * @param value
    *    the string to be truncated
    *    
    * @return the truncated argument
    */
   @Override
   protected String truncateValue(String value)
   {
         return RTRIM.matcher(value).replaceAll("");
   }

   /**
    * Remove trailing whitespaces if applicable
    * 
    * @param value
    *    the BaseDataType to be truncated
    *    
    * @return
    *    the truncated argument if the argument is of type character
    *    (nothing is required for non-character types)
    */
   @Override
   protected BaseDataType truncateValue(BaseDataType value)
   {
      if (value instanceof character)
      {
         return new character(truncateValue(value.toStringMessage()));
      }
      return value;
   }

   /**
    * Create new ControlSetItem
    * 
    * @param label
    *       item' label
    * @param value
    *       item's value
    *       
    * @return 
    *    new instance of the ControlSetItem
    */
   @Override
   protected ControlSetItem controlSetItem(character label, BaseDataType value) 
   {
      return new ControlSetItem(config.itemId++, 
            label.isUnknown() ? new character(" ") : (character)label.duplicate(), 
            truncateValue(value.duplicate())); 
   }
   
   /**
    * Truncate values of the provided items and remove invalid ones.
    * 
    * @param items
    *    items to be processed
    *    
    * @return
    *    valid items with truncated values
    */
   @Override
   protected ControlSetItem[] truncateValues(ControlSetItem[] items)
   {
      List<ControlSetItem> list = new ArrayList<>(items.length);
      for (ControlSetItem item : items)
      {
         if (invalid(item.getLabel(), item.getValue()))
         {
            continue;
         }
         list.add(controlSetItem(item.getLabel(), item.getValue()));
      }
      return list.toArray(new ControlSetItem[list.size()]);
   }
   
   /**
    * 
    * Checks if a value should be ignored (UNKNOWN for COMBO-BOX)
    *  
    * @param 
    *    value - value to be checked
    *    
    * @return <code>true</code> if the value should be ignored
    */
   @Override
   protected boolean ignore(BaseDataType value)
   {
      return value != null && "\"?\"".equals(value.toStringExport()); 
   }

   /**
    * Checks if a value should be ignored ("?" for the COMBO-BOX)
    *  
    * @param value
    *          value to be checked
    *    
    * @return <Code>true</code> if the value should be ignored
    */
   @Override
   protected boolean ignore(String value)
   {
      return "?".equals(value); 
   }
}
